sub SetCameraPosition( x, y, z)
sub GetCameraPosition(ByRef x, ByRef y, ByRef z)
sub TranslateCamera( x, y, z)
sub SetCameraRotation( x, y, z)
sub GetCameraRotation(ByRef x, ByRef y, ByRef z)
sub RotateCamera( x, y, z)
sub SetCameraFOV( fov )
function GetCameraFOV( )
sub SetCameraAspectRatio( aspect )
function GetCameraAspectRatio( )
sub SetCameraFarValue( zf )
function GetCameraFarValue( )
sub SetCameraNearValue( zn )
function GetCameraNearValue( )
Sub SetProjectionMatrix(matA, projection_type)
Sub GetProjectionMatrix(matA )
Sub GetWorldToViewportPosition(x, y, z, ByRef vx, ByRef vy)
